"""Application branding and release metadata."""

APP_NAME = "ExposoGraph"
APP_TAGLINE = "Carcinogen-Gene Knowledge Graph Builder"
CONTACT_EMAIL = "exposograph@kazilab.se"
DEVELOPED_BY = "Data analysis team @ KaziLab"
COPYRIGHT_HOLDER = "KaziLab"
APP_VERSION = "0.0.3"
